]> git.sesse.net Git - cubemap/blobdiff - stream.h
Use in-class initialization for making it harder to forget to set a default.
[cubemap] / stream.h
index d29b9246a06f76a776385bad4290f69b6cb171be..6c757a19ff29224f1049c2fc8554d4c24f36af0a 100644 (file)
--- a/stream.h
+++ b/stream.h
@@ -8,23 +8,18 @@
 #include <stdint.h>
 #include <sys/types.h>
 #include <sys/uio.h>
+#include <deque>
 #include <string>
 #include <vector>
 
-class MarkPool;
 class StreamProto;
 struct Client;
 
-enum StreamStartSuitability {
-       NOT_SUITABLE_FOR_STREAM_START,
-       SUITABLE_FOR_STREAM_START,
-};
-
 struct Stream {
        // Must be in sync with StreamConfig::Encoding.
        enum Encoding { STREAM_ENCODING_RAW = 0, STREAM_ENCODING_METACUBE };
 
-       Stream(const std::string &stream_id, size_t backlog_size, Encoding encoding);
+       Stream(const std::string &stream_id, size_t backlog_size, size_t prebuffering_bytes, Encoding encoding, Encoding src_encoding);
        ~Stream();
 
        // Serialization/deserialization.
@@ -51,6 +46,9 @@ struct Stream {
        // be Metacube, for reflecting to another Cubemap instance).
        Encoding encoding;
 
+       // What encoding we expect the incoming data to be in (usually Metacube).
+       Encoding src_encoding;
+
        // The stream data itself, stored in a circular buffer.
        //
        // We store our data in a file, so that we can send the data to the
@@ -64,13 +62,21 @@ struct Stream {
        // How many bytes <data_fd> can hold (the buffer size).
        size_t backlog_size;
 
+       // How many bytes we need to have in the backlog before we start
+       // sending (in practice, we will then send all of them at once,
+       // and then start sending at the normal rate thereafter).
+       // This is basically to force a buffer on the client, which can help
+       // if the client expects us to be able to fill up the buffer much
+       // faster than realtime (ie., it expects a static file).
+       size_t prebuffering_bytes;
+
        // How many bytes this stream have received. Can very well be larger
        // than <backlog_size>, since the buffer wraps.
-       size_t bytes_received;
+       size_t bytes_received = 0;
 
-       // The last point in the stream that is suitable to start new clients at
-       // (after having sent the header). -1 if no such point exists yet.
-       ssize_t last_suitable_starting_point;
+       // A list of points in the stream that is suitable to start new clients at
+       // (after having sent the header). Empty if no such point exists yet.
+       std::deque<size_t> suitable_starting_points;
        
        // Clients that are in SENDING_DATA, but that we don't listen on,
        // because we currently don't have any data for them.
@@ -81,19 +87,16 @@ struct Stream {
        // <sleeping_clients>).
        std::vector<Client *> to_process;
 
-       // What pool to fetch marks from, or NULL.
-       MarkPool *mark_pool;
-
        // Maximum pacing rate for the stream.
-       int pacing_rate;
+       uint32_t pacing_rate = ~0U;
 
        // Queued data, if any. Protected by <queued_data_mutex>.
        // The data pointers in the iovec are owned by us.
-       std::vector<iovec> queued_data;
-
-       // Index of the last element in queued_data that is suitable to start streaming at.
-       // -1 if none. Protected by <queued_data_mutex>.
-       int queued_data_last_starting_point;
+       struct DataElement {
+               iovec data;
+               uint16_t metacube_flags;
+       };
+       std::vector<DataElement> queued_data;
 
        // Put client to sleep, since there is no more data for it; we will on
        // longer listen on POLLOUT until we get more data. Also, it will be put
@@ -102,7 +105,7 @@ struct Stream {
 
        // Add more data to <queued_data>, adding Metacube headers if needed.
        // Does not take ownership of <data>.
-       void add_data_deferred(const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start);
+       void add_data_deferred(const char *data, size_t bytes, uint16_t metacube_flags);
 
        // Add queued data to the stream, if any.
        // You should hold the owning Server's <mutex>.
@@ -113,8 +116,14 @@ private:
 
        // Adds data directly to the stream file descriptor, without adding headers or
        // going through <queued_data>.
+       // You should hold the owning Server's <mutex>, and probably call
+       // remove_obsolete_starting_points() afterwards.
+       void add_data_raw(const std::vector<DataElement> &data);
+
+       // Remove points from <suitable_starting_points> that are no longer
+       // in the backlog.
        // You should hold the owning Server's <mutex>.
-       void add_data_raw(const std::vector<iovec> &data);
+       void remove_obsolete_starting_points();
 };
 
 #endif  // !defined(_STREAM_H)